@@ -220,17 +220,22 @@ network_changed_cb (GObject     *gobject,
 {
   GNetworkConnectivity connectivity;
   gboolean network_available;
-  GHashTableIter iter;
   GrlSource *current_source;
+  GList *sources, *l;
 
   GRL_DEBUG ("Network availability changed");
   get_connectivity (registry, &connectivity, &network_available);
 
-  if (!network_available) {
-    g_hash_table_iter_init (&iter, registry->priv->sources);
+  sources = g_hash_table_get_values (registry->priv->sources);
+  if (!sources)
+    return;
 
-    while (g_hash_table_iter_next (&iter, NULL, (gpointer *) &current_source)) {
-      const char **tags = grl_source_get_tags (current_source);
+  if (!network_available) {
+    for (l = sources; l != NULL; l = l->next) {
+      const char **tags;
+
+      current_source = l->data;
+      tags = grl_source_get_tags (current_source);
 
       if (!tags)
         continue;
@@ -245,9 +250,15 @@ network_changed_cb (GObject     *gobject,
       }
     }
   } else {
-    g_hash_table_iter_init (&iter, registry->priv->sources);
-    while (g_hash_table_iter_next (&iter, NULL, (gpointer *) &current_source)) {
-      const char **tags = grl_source_get_tags (current_source);
+    GList *to_add, *to_remove;
+
+    to_add = to_remove = NULL;
+
+    for (l = sources; l != NULL; l = l->next) {
+      const char **tags;
+
+      current_source = l->data;
+      tags = grl_source_get_tags (current_source);
 
       if (!tags)
         continue;
@@ -256,8 +267,7 @@ network_changed_cb (GObject     *gobject,
           SOURCE_IS_INVISIBLE(current_source)) {
         GRL_DEBUG ("Local network became available for '%s', showing",
                    grl_source_get_id (current_source));
-        SET_INVISIBLE_SOURCE(current_source, FALSE);
-        g_signal_emit (registry, registry_signals[SIG_SOURCE_ADDED], 0, current_source);
+        to_add = g_list_prepend (to_add, current_source);
       }
 
       if (g_strv_contains (tags, INTERNET_NET_TAG) &&
@@ -265,8 +275,7 @@ network_changed_cb (GObject     *gobject,
           SOURCE_IS_INVISIBLE(current_source)) {
         GRL_DEBUG ("Internet became available for '%s', showing",
                    grl_source_get_id (current_source));
-        SET_INVISIBLE_SOURCE(current_source, FALSE);
-        g_signal_emit (registry, registry_signals[SIG_SOURCE_ADDED], 0, current_source);
+        to_add = g_list_prepend (to_add, current_source);
       }
 
       if (g_strv_contains (tags, INTERNET_NET_TAG) &&
@@ -274,11 +283,24 @@ network_changed_cb (GObject     *gobject,
           !SOURCE_IS_INVISIBLE(current_source)) {
         GRL_DEBUG ("Internet became unavailable for '%s', hiding",
                    grl_source_get_id (current_source));
-        SET_INVISIBLE_SOURCE(current_source, TRUE);
-        g_signal_emit (registry, registry_signals[SIG_SOURCE_REMOVED], 0, current_source);
+        to_remove = g_list_prepend (to_remove, current_source);
       }
     }
+
+    for (l = to_add; l != NULL; l = l->next) {
+      SET_INVISIBLE_SOURCE(l->data, FALSE);
+      g_signal_emit (registry, registry_signals[SIG_SOURCE_ADDED], 0, l->data);
+    }
+    g_list_free (to_add);
+
+    for (l = to_remove; l != NULL; l = l->next) {
+        SET_INVISIBLE_SOURCE(l->data, TRUE);
+        g_signal_emit (registry, registry_signals[SIG_SOURCE_REMOVED], 0, l->data);
+    }
+    g_list_free (to_remove);
   }
+
+  g_list_free (sources);
 }
 
 static void
@@ -380,9 +402,25 @@ set_source_rank (GrlRegistry *registry, GrlSource *source)
   rank =
     GPOINTER_TO_INT (g_hash_table_lookup (registry->priv->ranks,
                                           grl_source_get_id (source)));
+
+  if (!rank) {
+    GHashTableIter iter;
+    gpointer key, value;
+
+    g_hash_table_iter_init(&iter, registry->priv->ranks);
+
+    while (g_hash_table_iter_next (&iter, &key, &value)) {
+      if (g_pattern_match_simple (key, grl_source_get_id (source))) {
+        rank = GPOINTER_TO_INT (value);
+        break;
+      }
+    }
+  }
+
   if (!rank) {
     rank = GRL_RANK_DEFAULT;
   }
+
   g_object_set (source, "rank", rank, NULL);
   GRL_DEBUG ("Source rank '%s' : %d", grl_source_get_id (source), rank);
 }
@@ -625,7 +663,7 @@ grl_registry_register_metadata_key_for_type (GrlRegistry *registry,
                                    G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE);
     } else {
       GRL_WARNING ("'%s' is being ignored as G_TYPE '%s' is not being handled",
-                   key_name, G_VALUE_TYPE_NAME (type));
+                   key_name, g_type_name (type));
       return GRL_METADATA_KEY_INVALID;
     }
   }
@@ -1983,6 +2021,28 @@ grl_registry_add_config (GrlRegistry *registry,
   return TRUE;
 }
 
+static void
+get_plugin_and_source_from_group_name (const gchar  *group_name,
+                                       gchar       **plugin_name,
+                                       gchar       **source_name)
+{
+  gchar *plugin = g_strdup (group_name);
+  gchar **arr;
+
+  *plugin_name = *source_name = NULL;
+
+  plugin = g_strstrip (plugin);
+  arr = g_strsplit (plugin, " ", 2);
+  g_free (plugin);
+
+  *plugin_name = g_strstrip (arr[0]);
+
+  if (arr[1] != NULL)
+    *source_name = g_strstrip (arr[1]);
+
+  g_free (arr);
+}
+
 static void
 add_config_from_keyfile (GKeyFile    *keyfile,
 			 GrlRegistry *registry)
@@ -1990,26 +2050,35 @@ add_config_from_keyfile (GKeyFile    *keyfile,
   GrlConfig *config;
   gchar **key;
   gchar **keys;
-  gchar **plugin;
+  gchar **groupname;
   gchar **plugins;
   gchar *value;
 
   /* Look up for defined plugins */
   plugins = g_key_file_get_groups (keyfile, NULL);
-  for (plugin = plugins; *plugin; plugin++) {
-    config = grl_config_new (*plugin, NULL);
+  for (groupname = plugins; *groupname; groupname++) {
+    gchar *plugin_name, *source_name;
+
+    get_plugin_and_source_from_group_name (*groupname, &plugin_name, &source_name);
+
+    config = grl_config_new (plugin_name, source_name);
 
     /* Look up configuration keys for this plugin */
-    keys = g_key_file_get_keys (keyfile, *plugin, NULL, NULL);
+    keys = g_key_file_get_keys (keyfile, *groupname, NULL, NULL);
     for (key = keys; *key; key++) {
-      value = g_key_file_get_string (keyfile, *plugin, *key, NULL);
+      value = g_key_file_get_string (keyfile, *groupname, *key, NULL);
       if (value) {
+        GRL_DEBUG ("Config found: %s : %s : %s", plugin_name,
+                   source_name != NULL ? source_name : plugin_name,
+                   *key);
         grl_config_set_string (config, *key, value);
         g_free (value);
       }
     }
     grl_registry_add_config (registry, config, NULL);
     g_strfreev (keys);
+    g_free (source_name);
+    g_free (plugin_name);
   }
   g_strfreev (plugins);
 }
