@@ -41,6 +41,7 @@ registry_load_error_handler (const gchar *log_domain,
       CHECK_MESSAGE ("Grilo", "Configuration not provided") ||
       CHECK_MESSAGE ("Grilo", "Missing configuration") ||
       CHECK_MESSAGE ("Grilo", "Could not open plugin directory") ||
+      CHECK_MESSAGE ("Grilo", "Error opening directory") ||
       CHECK_MESSAGE ("Grilo", "Could not read XML file")) {
     return FALSE;
   }
@@ -81,9 +82,13 @@ static void
 registry_load (RegistryFixture *fixture, gconstpointer data)
 {
   gboolean res;
+  GError *err = NULL;
 
-  res = grl_registry_load_all_plugins (fixture->registry, TRUE, NULL);
-  g_assert_cmpint (res, ==, TRUE);
+  res = grl_registry_load_all_plugins (fixture->registry, TRUE, &err);
+  if (!res) {
+    g_assert_error (err, GRL_CORE_ERROR, GRL_CORE_ERROR_LOAD_PLUGIN_FAILED);
+    g_test_skip ("No sources loaded, skipping test");
+  }
 }
 
 static void
@@ -96,6 +101,10 @@ registry_unregister (RegistryFixture *fixture, gconstpointer data)
   g_test_bug ("627207");
 
   sources = grl_registry_get_sources (fixture->registry, FALSE);
+  if (sources == NULL) {
+    g_test_skip ("No sources loaded, skipping test");
+    return;
+  }
 
   for (sources_iter = sources, i = 0; sources_iter;
       sources_iter = g_list_next (sources_iter), i++) {
